bl_info = {
	"name": "Train Fever / Transport Fever (.mdl)",
	"author": "Merk, eis_os, nary_believe", # Licensed under CC BY-NC-SA 2.0 (https://creativecommons.org/licenses/by-nc-sa/2.0/) 
	"version": (0, 6, 0),
	"blender": (2, 70, 0),
	"category": "Import-Export",
}

tf_plugins = {}

import os
import os.path
import sys

script_basepath = os.path.dirname(__file__)
addonID = "io_mesh_mdl"

filelist = os.listdir(script_basepath+"/plugins")
for file in filelist:
	if os.path.isfile(script_basepath+"/plugins/"+file):
		if file.endswith(".py"):
			tf_plugins[file[:-3]] = None
	
if sys.version_info[0] >= 3 and sys.version_info[1] >= 3:
	import importlib
	for plugin in tf_plugins.keys():
		try:
			tf_plugins[plugin] = importlib.import_module(addonID + ".plugins."+plugin)
		except:
			print("Error while loading plugin: "+plugin)
else:
	for plugin in tf_plugins.keys():
		try:
			mod = imp.find_module(addonID + "."+plugin)
			try:
				tf_plugins[plugin] = imp.load_module(addonID + "."+plugin, mod[0], mod[1], mod[2])
			except:
				print("Error while loading plugin: "+plugin)
			finally:
				mod[0].close()
		except:
			pass
			
if "bpy" in locals():
	print("Reloading io_mesh_mdl __init__.py")
	import imp
	imp.reload(tf_mesh_mdl)
else:
	import bpy
	from io_mesh_mdl import tf_mesh_mdl

#settings
from bpy.types import AddonPreferences, Operator, PropertyGroup, UIList

from io_mesh_mdl.tf_util.base_functions import TF_base_functions
TF_base_functions.addonID = addonID

class TFCheck_for_updates(bpy.types.Operator):
	"""Check for Updates"""
	bl_idname = "wm.tf_check_for_update"
	bl_label = "Update Check"
	bl_options = {'INTERNAL'}

	update = bpy.props.BoolProperty(default = False)
	website = bpy.props.StringProperty()
	
	def invoke(self, context, event):
		self.update, self.website = TF_base_functions.check_for_updates()
		context.window_manager.invoke_props_dialog(self, 400)
		return {'RUNNING_MODAL'}
	
	def draw(self, context):
		if self.update:
			self.layout.label("A new version of the addon is available.")
			if self.website != "":
				self.layout.label("Do you want to visit the download page?")
		else:
			self.layout.label("No update available.")

	def execute(self, context):
		if self.update and self.website != "":
			bpy.ops.wm.url_open(url=self.website)
		
		return {'FINISHED'}

class TFPlugin_expand(Operator):
	bl_label = "Show/Hide Plugin Details"
	bl_description = "Toggle Plugin Details"
	bl_idname = "wm.tfplugin_expand"
	
	plugin_id = bpy.props.StringProperty(name="Plugin ID", default="")
	
	def invoke(self, context, event):
		for plugin in context.user_preferences.addons[addonID].preferences.plugins:
			if plugin.id == self.plugin_id:
				plugin.expanded = not plugin.expanded
				break
		return {'FINISHED'}

class TFPlugin_enable(Operator):
	bl_label = "Activate/Deactivate Plugin"
	bl_description = "Activate/Deactivate Plugin"
	bl_idname = "wm.tfplugin_enable"
	
	plugin_id = bpy.props.StringProperty(name="Plugin ID", default="")
	
	def invoke(self, context, event):
		for plugin in context.user_preferences.addons[addonID].preferences.plugins:
			if plugin.id == self.plugin_id:
				if plugin.enabled:
					if self.plugin_id in tf_plugins:
						tf_plugins[self.plugin_id].unregister()
					plugin.enabled = False
				else:
					if self.plugin_id in tf_plugins:
						tf_plugins[self.plugin_id].register()
					plugin.enabled = True
				break
		return {'FINISHED'}

class TFAddonPluginSettings(PropertyGroup):
	enabled = bpy.props.BoolProperty(name="Enabled", default=True)
	id = bpy.props.StringProperty(name="Plugin ID", default="")
	name = bpy.props.StringProperty(name="Plugin Name", default="")
	description = bpy.props.StringProperty(name="Plugin Desc", default="")
	author = bpy.props.StringProperty(name="Plugin Author", default="")
	version = bpy.props.StringProperty(name="Plugin Version", default="")
	expanded = bpy.props.BoolProperty(default=False)

class TFAddonPluginUIList(bpy.types.UIList):
	use_filter_show = False

	def draw_item(self, context, layout, data, item, icon, active_data, active_propname, index):
		maincol = layout.column()
		box = maincol.box()
		col = box.column()
		
		row = col.row()
		row.operator("wm.tfplugin_expand", icon='TRIA_DOWN' if item.expanded else 'TRIA_RIGHT', text="", emboss=False).plugin_id = item.id
		sub = row.row()
		sub.active = (item.enabled and tf_plugins[item.id] != None)
		if item.name:
			sub.label(text=item.name)
		else:
			sub.label(text=item.id)
		
		if not tf_plugins[item.id]:
			row.label(icon='ERROR')
		else:
			row.operator("wm.tfplugin_enable", icon='CHECKBOX_HLT' if item.enabled else 'CHECKBOX_DEHLT', text="", emboss=False).plugin_id = item.id
			
		if item.expanded:
			if tf_plugins[item.id]:
				if item.description:
					split = col.row().split(percentage=0.15)
					split.label(text="Description:")
					split.label(text=item.description)
				if item.id in tf_plugins:
					split = col.row().split(percentage=0.15)
					split.label(text="File:")
					split.label(text=tf_plugins[item.id].__file__.replace('\\', '/'), translate=False)
				if item.author:
					split = col.row().split(percentage=0.15)
					split.label(text="Author:")
					split.label(text=item.author, translate=False)
				if item.version:
					split = col.row().split(percentage=0.15)
					split.label(text="Version:")
					split.label(text=item.version, translate=False)
			else:
				col.row().label(icon='ERROR', text="An error occured while loading this plugin. Therefore this plugin can't be enabled")
		
		maincol.separator()

def update_branch(self, context):
	TF_base_functions.user_prefs["update_branch"] = self.update_branch
	if self.update_auto_check:
		TF_base_functions.update = TF_base_functions.check_for_updates()

def update_auto_check(self, context):
	TF_base_functions.user_prefs["update_branch"] = self.update_branch
	if self.update_auto_check:
		TF_base_functions.update = TF_base_functions.check_for_updates()

class TrainFeverAddonPreferences(AddonPreferences):
	bl_idname = addonID

	lua_scripts_basepath = bpy.props.StringProperty(
			name="Train Fever Path",
			subtype='DIR_PATH',
			description="Path to TrainFever.exe, used to find required Lua-scripts.",
			default="",
		)
	lua_scripts_basepath_tpf = bpy.props.StringProperty(
			name="Transport Fever Path",
			subtype='DIR_PATH',
			description="Path to TransportFever.exe, used to find required Lua-scripts.",
			default="",
		)
	lua_scripts_basepath_tpf2 = bpy.props.StringProperty(
			name="Transport Fever 2 Path",
			subtype='DIR_PATH',
			description="Path to TransportFever2.exe, used to find required Lua-scripts.",
			default="",
		)
	zip_extract_dir = bpy.props.StringProperty(
			name="Temporary File Path",
			subtype='DIR_PATH',
			description="Path to a folder, used for extracts from models.zip.",
			default="",
		)
	error_level = bpy.props.IntProperty(
			name="Error Level",
			description="0: Only display errors which cause an incomplete import/export (e.g. file not found) and/or will likely crash the game. 1: Display all errors and hints.",
			default=0,
			min=0,
			max=1,
		)
	debug = bpy.props.BoolProperty(
			name="Debug Mode",
			description="More status messages.",
			default=False,
		)
	dds_converter = bpy.props.StringProperty(
			name="DDS Converter",
			subtype = 'FILE_PATH',
			description= "Programm for converting .tga textures to .dds textures (only NVIDIA Texture Tools supported).",
			default="",
		)
	plugins = bpy.props.CollectionProperty(
			type=TFAddonPluginSettings
		)
	plugins_activeidx = bpy.props.IntProperty(
			name="Number of selected Plugin",
			default=0,
		)
	update_auto_check = bpy.props.BoolProperty(
			name="Auto Check for Updates",
			description= "Check for updates every time the addon is loaded.",
			default=False,
			update = update_auto_check
		)
	update_branch = bpy.props.EnumProperty(
			name = "Update Branch",
			description = "Where to check for updates.",
			items = [
				("stable", "Stable", "", 0),
				("testing", "Testing", "", 1),
			],
			update = update_branch
		)

	def draw(self, context):
		layout = self.layout
		row = layout.row(True)
		if self.update_auto_check and TF_base_functions.update[0]:
			row.operator("wm.url_open", text="Update available!").url=TF_base_functions.update[1]
		else:
			row.operator("wm.tf_check_for_update", text="Check for Updates")
		row.prop(self, "update_auto_check", text="Auto Check", toggle=True)
		row.prop(self, "update_branch", text = "")
		split = layout.split(percentage=0.25)
		split.label("Train Fever Directory:")
		split.prop(self, "lua_scripts_basepath", text="")
		split = layout.split(percentage=0.25)
		split.label("Transport Fever Directory:")
		split.prop(self, "lua_scripts_basepath_tpf", text="")
		split = layout.split(percentage=0.25)
		split.label("Transport Fever 2 Directory:")
		split.prop(self, "lua_scripts_basepath_tpf2", text="")
		split = layout.split(percentage=0.25)
		split.label("Temporary File Directory:")
		split.prop(self, "zip_extract_dir", text="")
		split = layout.split(percentage=0.25)
		split.label("Error Level:")
		split.prop(self, "error_level", text="")
		split = layout.split(percentage=0.25)
		split.label("Debug Mode:")
		split.prop(self, "debug", text="")
		#split = layout.split(percentage=0.25)
		#split.label("DDS Converter:")
		#split.prop(self, "dds_converter", text="")
		layout.label("Plugins:")
		layout.template_list(
				listtype_name="TFAddonPluginUIList",
				dataptr=self,
				propname="plugins",
				active_dataptr=self,
				active_propname="plugins_activeidx",
				rows = 1,
				maxrows = 5
			)

def find_lua_basepath():
	if (bpy.context.user_preferences.addons.find(addonID) > -1):
		lua_scripts_basepath = bpy.context.user_preferences.addons[addonID].preferences.lua_scripts_basepath
	else:
		lua_scripts_basepath = ''
	if lua_scripts_basepath == '':
		running_os = sys.platform
		if running_os == 'win32' or running_os == 'cygwin':
			import winreg
			reg = winreg.ConnectRegistry(None, winreg.HKEY_LOCAL_MACHINE)
			key = None
			try:
				key = winreg.OpenKey(reg, r"SOFTWARE\Microsoft\Windows\CurrentVersion\Uninstall\Steam App 304730")
			except:
				pass
			try:
				key = winreg.OpenKey(reg, r"SOFTWARE\Wow6432Node\Microsoft\Windows\CurrentVersion\Uninstall\Steam App 304730")
			except:
				pass
			if key:
				for i in range(winreg.QueryInfoKey(key)[1]):
					if winreg.EnumValue(key, i)[0] == "InstallLocation":
						lua_scripts_basepath = winreg.EnumValue(key, i)[1].replace('\\', '/')
			if not os.path.isfile(lua_scripts_basepath + '/TrainFever.exe'):
				lua_scripts_basepath = ''
		elif running_os.startswith('linux'):
			lua_scripts_basepath = os.path.expanduser('~') + "/.local/share/Steam/SteamApps/common/Train Fever"
			if not os.path.isfile(lua_scripts_basepath + '/TrainFever'):
				lua_scripts_basepath = ''
		elif running_os == 'darwin':
			lua_scripts_basepath = os.path.expanduser('~') + "/Library/Application Support/Steam/SteamApps/common/Train Fever"
			if not os.path.isfile(lua_scripts_basepath + '/TrainFever'):
				lua_scripts_basepath = ''
		
		if lua_scripts_basepath == '':
			print('Failed to find "TrainFever.exe"! Import of models which use additional scripts may not work properly.')
	bpy.context.user_preferences.addons[addonID].preferences.lua_scripts_basepath = lua_scripts_basepath

def register():
	bpy.utils.register_class(TFCheck_for_updates)
	bpy.utils.register_class(TFAddonPluginSettings)
	bpy.utils.register_class(TFAddonPluginUIList)
	bpy.utils.register_class(TFPlugin_expand)
	bpy.utils.register_class(TFPlugin_enable)
	bpy.utils.register_class(TrainFeverAddonPreferences)
	find_lua_basepath()
	
	sorted_plugins = []
	
	for plugin in tf_plugins.keys():
		plugin_pref = None
		for plugin_prefs in bpy.context.user_preferences.addons[addonID].preferences.plugins:
			if plugin_prefs.id == plugin:
				plugin_pref = plugin_prefs
		
		if not plugin_pref:
			plugin_pref = bpy.context.user_preferences.addons[addonID].preferences.plugins.add()
			plugin_pref.id = plugin
		
		if tf_plugins[plugin]:
			for attr in tf_plugins[plugin].plugin_info.keys():
				if hasattr(plugin_pref, attr):
					setattr(plugin_pref, attr, tf_plugins[plugin].plugin_info[attr])
			i = 0
			plugin_info = tf_plugins[plugin].plugin_info
			while "order" in plugin_info and plugin_info["order"] > -1 and i < len(sorted_plugins):
				if plugin_info["order"] < sorted_plugins[i]["order"] or sorted_plugins[i]["order"] == -1:
					sorted_plugins.insert(i, {
						"order": plugin_info["order"],
						"name": plugin,
						"plugin": tf_plugins[plugin],
						"preferences": plugin_pref,
					})
					break
				i += 1
			
			if i == len(sorted_plugins) or "order" not in plugin_info:
				new_plugin = {
					"name": plugin,
					"preferences": plugin_pref,
					"plugin": tf_plugins[plugin],
				}
				if "order" in plugin_info:
					new_plugin["order"] = plugin_info["order"]
				else:
					new_plugin["order"] = -1
				
				sorted_plugins.append(new_plugin)

	for plugin in sorted_plugins:
		if plugin["preferences"].enabled and hasattr(plugin["plugin"], "pre_register"):
			try:
				plugin["plugin"].pre_register()
			except:
				print("Error while pre registering plugin: "+plugin["name"])
	
	tf_mesh_mdl.register()

	for plugin in sorted_plugins:
		if plugin["preferences"].enabled:
			try:
				plugin["plugin"].register()
			except:
				print("Error while registering plugin: "+plugin["name"])

	
	i = 0
	while i < len(bpy.context.user_preferences.addons[addonID].preferences.plugins):
		plugin = bpy.context.user_preferences.addons[addonID].preferences.plugins[i]
		if not plugin.id in tf_plugins:
			bpy.context.user_preferences.addons[addonID].preferences.plugins.remove(i)
		else:
			i = i + 1
	
def unregister():
	print("unregister io_mesh_mdl")
	for plugin in tf_plugins.keys():
		if tf_plugins[plugin]:
			plugin_pref = None
			for plugin_prefs in bpy.context.user_preferences.addons[addonID].preferences.plugins:
				if plugin_prefs.id == plugin:
					plugin_pref = plugin_prefs
			if plugin_pref and plugin_pref.enabled:
				if hasattr(tf_plugins[plugin], "pre_unregister"):
					try:
						tf_plugins[plugin].pre_unregister()
					except:
						print("Error while pre unregistering plugin: "+plugin)
				try:
					tf_plugins[plugin].unregister()
				except:
					print("Error while unregistering plugin: "+plugin)
	
	tf_mesh_mdl.unregister()
	bpy.utils.unregister_class(TrainFeverAddonPreferences)
	bpy.utils.unregister_class(TFPlugin_enable)
	bpy.utils.unregister_class(TFPlugin_expand)
	bpy.utils.unregister_class(TFAddonPluginUIList)
	bpy.utils.unregister_class(TFAddonPluginSettings)
	bpy.utils.unregister_class(TFCheck_for_updates)
