import bpy
from ..tf_util.base_functions import TF_base_functions
from ..tf_util import material_util

plugin_info = {
	"name": "Materials and MatConfigs",
	"description": "Support for multiple materials per submesh and matConfigs",
	"author": "Merk",
	"version": "0.7.0",
	"order": 20,
}
name_lists = [("Default", "Default", "Default")]

def get_materials_by_slot(self, context):
	obj = self.id_data
	mat_list = []
	if not hasattr(obj, "data"):
		mat_slot = None
		for j in range(len(obj.tf_matslots)):
			if obj.tf_matslots[j].slot_index == self.slot_id:
				mat_slot = obj.tf_matslots[j]
		if mat_slot:
			for mat in mat_slot.tf_mats:
				list_entry = (mat.mat_name, mat.mat_name, mat.mat_name)
				if not list_entry in name_lists:
					name_lists.append(list_entry)
				if not list_entry in mat_list:
					mat_list.append(name_lists[name_lists.index(list_entry)])
		for parent_obj in bpy.data.objects:
			if parent_obj.type == 'MESH' and parent_obj.data == obj and self.slot_id < len(parent_obj.material_slots):
				list_entry = (parent_obj.material_slots[self.slot_id].material.name, parent_obj.material_slots[self.slot_id].material.name, parent_obj.material_slots[self.slot_id].material.name)
				if not list_entry in name_lists:
					name_lists.append(list_entry)
				if not name_lists[name_lists.index(list_entry)] in mat_list:
					mat_list = [name_lists[name_lists.index(list_entry)]] + mat_list
	elif obj.subtype in {'group', 'lod'}:
		child = None
		if "child_name" in self and self["child_name"] in bpy.data.objects:
			if bpy.data.objects[self["child_name"]] in obj.children:
				child = bpy.data.objects[self["child_name"]] 
		if child:
			if hasattr(child, "tf_matConfigs"):
				for matConfig in child.tf_matConfigs:
					list_entry = (matConfig.matConfig_name, matConfig.matConfig_name, matConfig.matConfig_name)
					if not list_entry in name_lists:
						name_lists.append(list_entry)
					if not name_lists[name_lists.index(list_entry)] in mat_list:
						mat_list.append(name_lists[name_lists.index(list_entry)])
			if hasattr(child, "data") and hasattr(child.data, "tf_matConfigs"):
				for matConfig in child.data.tf_matConfigs:
					list_entry = (matConfig.matConfig_name, matConfig.matConfig_name, matConfig.matConfig_name)
					if not list_entry in name_lists:
						name_lists.append(list_entry)
					if not name_lists[name_lists.index(list_entry)] in mat_list:
						mat_list.append(name_lists[name_lists.index(list_entry)])
	if len(mat_list) == 0:
		mat_list.append(name_lists[0])
	return mat_list

def get_mat_index_by_slot(self, slot_name):
	obj = self.id_data
	mat_index = 0
	if not hasattr(obj, "data"):
		mat_found = False
		base_mats_in_list = 0
		base_mat_names = []

		for parent_obj in bpy.data.objects:
			if parent_obj.type == 'MESH' and parent_obj.data == obj and self.slot_id < len(parent_obj.material_slots) and parent_obj.material_slots[self.slot_id].material:
				if not parent_obj.material_slots[self.slot_id].material.name in base_mat_names:
					base_mat_names.append(parent_obj.material_slots[self.slot_id].material.name)

		mat_slot = None
		for j in range(len(obj.tf_matslots)):
			if obj.tf_matslots[j].slot_index == self.slot_id:
				mat_slot = obj.tf_matslots[j]

		if mat_slot:
			i = 0
			for mat in mat_slot.tf_mats:
				if not mat_found and mat.mat_name == slot_name:
					mat_index = i
					mat_found = True
				if mat.mat_name in base_mat_names:
					base_mats_in_list += 1
				i += 1

			if mat_found:
				mat_index += (len(base_mat_names) - base_mats_in_list)
			elif slot_name in base_mat_names:
				mat_index == base_mat_names.index(slot_name)

	elif obj.subtype in {'group', 'lod'}:
		child = None
		if "child_name" in self and self["child_name"] in bpy.data.objects:
			if bpy.data.objects[self["child_name"]] in obj.children:
				child = bpy.data.objects[self["child_name"]] 
		if child:
			i = 0
			if hasattr(child, "data") and hasattr(child.data, "tf_matConfigs"):
				# Meshes
				for matConfig in child.data.tf_matConfigs:
					if matConfig.matConfig_name == slot_name:
						return i
					i += 1
			else:
				# Groups/Lods
				for matConfig in child.tf_matConfigs:
					if matConfig.matConfig_name == slot_name:
						return i
					i += 1
				
	return mat_index

def get_slot_num(obj):
	if not hasattr(obj, "data"):
		return len(obj.materials)
	else:
		return len(obj.children)

# Material settings classes
class TFMatSetting(bpy.types.PropertyGroup):
	mat_name = bpy.props.StringProperty()

	@classmethod
	def register(cls):
		TFMatSlotSetting.tf_mats = bpy.props.CollectionProperty(
						type = cls,
					)
		TFMatSlotSetting.tf_mats_activeidx = bpy.props.IntProperty(
						min = 0,
						default = 0,
					)

	@classmethod
	def unregister(cls):
		del TFMatSlotSetting.tf_mats
		del TFMatSlotSetting.tf_mats_activeidx

class TFMatAddOp(bpy.types.Operator):
	bl_label = "Add material property"
	bl_idname = "collection.tfmat_add"

	index = bpy.props.IntProperty(min = 0)

	def invoke(self, context, event):
		obj = context.object.data
		collection = obj.tf_matslots[self.index].tf_mats
		item = collection.add()
		obj.tf_matslots[self.index].tf_mats_activeidx = len(collection) - 1
		return {'FINISHED'}

class TFMatRemoveOp(bpy.types.Operator):
	bl_label = "Remove material property"
	bl_idname = "collection.tfmat_remove"

	index = bpy.props.IntProperty(min = 0)

	def invoke(self, context, event):
		obj = context.object.data
		collection = obj.tf_matslots[self.index].tf_mats
		index = obj.tf_matslots[self.index].tf_mats_activeidx
		collection.remove(index)
		if index > 0:
			obj.tf_matslots[self.index].tf_mats_activeidx = index-1
		return {'FINISHED'}

class TFMatSettingsItemUIList(bpy.types.UIList):
	def draw_item(self, context, layout, data, item, icon, active_data, active_propname, index):
		layout.prop_search(item, "mat_name", bpy.data, "materials", text="Name")

class TFMatSlotSetting(bpy.types.PropertyGroup):
	slot_index = bpy.props.IntProperty(min = 0)

	@classmethod
	def register(cls):
		bpy.types.Mesh.tf_matslots = bpy.props.CollectionProperty(
						type = cls,
					)
		bpy.types.Mesh.tf_matslots_activeidx = bpy.props.IntProperty(
						min = 0,
						default = 0,
					)

	@classmethod
	def unregister(cls):
		del bpy.types.Mesh.tf_matslots
		del bpy.types.Mesh.tf_matslots_activeidx

class TFMatSlotAddOp(bpy.types.Operator):
	bl_label = "Add materialslot"
	bl_idname = "collection.tfmatslot_add"

	def invoke(self, context, event):
		obj = context.object.data
		collection = obj.tf_matslots
		item = collection.add()
		item.slot_index = len(collection)-1
		return {'FINISHED'}

class TFMatSlotRemoveOp(bpy.types.Operator):
	bl_label = "Remove materialslot"
	bl_idname = "collection.tfmatslot_remove"

	def invoke(self, context, event):
		obj = context.object.data
		collection = obj.tf_matslots
		index = obj.tf_matslots_activeidx
		collection.remove(index)
		if index > 0:
			obj.tf_matslots_activeidx = index-1
		return {'FINISHED'}

class TFMatSlotSettingsItemUIList(bpy.types.UIList):
	def draw_item(self, context, layout, data, item, icon, active_data, active_propname, index):
		row = layout.row()
		row.prop(item, "slot_index", "Slot Index")
		if item.slot_index < len(context.object.material_slots) and context.object.material_slots[item.slot_index].name:
			row.label("Current Material: \"%s\"" % context.object.material_slots[item.slot_index].name)
		else:
			row.label("Material slot %s" % str(item.slot_index))
		row.template_list(
			listtype_name="TFMatSettingsItemUIList",
            list_id = data.name + "tf_matslot" + str(index),
			dataptr=item,
			propname="tf_mats",
			active_dataptr=item,
			active_propname="tf_mats_activeidx",
			rows = 1,
			maxrows = 2
		)
		col = row.column(align=True)
		col.operator("collection.tfmat_add", icon="ZOOMIN", text="").index = index
		col.operator("collection.tfmat_remove", icon="ZOOMOUT", text="").index = index

# MatConfig settings classes
class TFMatConfigSlot(bpy.types.PropertyGroup):
	def get_children_names(self, context):
		child_list = []
		if hasattr(self.id_data, "children"):
			for child in self.id_data.children:
				list_entry = (child.name, child.name, child.name)
				if not list_entry in name_lists:
					name_lists.append(list_entry)
				if not name_lists[name_lists.index(list_entry)] in child_list:
					child_list.append(name_lists[name_lists.index(list_entry)])
		if len(child_list) == 0 or not "child_name" in self or self["child_name"] == "":
			child_list.append(("Unknown", "Unknown", "Unknown"))
		return child_list

	def get_matname(self):
		materials = get_materials_by_slot(self, bpy.context)
		if "slot_name" in self:
			if (self["slot_name"], self["slot_name"], self["slot_name"]) in materials:
				self.slot_num = materials.index((self["slot_name"], self["slot_name"], self["slot_name"]))
				return self.slot_num
			elif self.slot_num < len(materials):
				return self.slot_num
		return 0

	def set_matname(self, value):
		materials = get_materials_by_slot(self, bpy.context)
		self["slot_name"] = materials[value][0]
		self.slot_num = value

	def get_child_name(self):
		child_list = self.get_children_names(bpy.context)
		if "child_name" in self and (self["child_name"], self["child_name"], self["child_name"]) in child_list:
			return child_list.index((self["child_name"], self["child_name"], self["child_name"]))
		elif self.slot_id < len(child_list):
			return self.slot_id
		return len(child_list)-1

	def set_child_name(self, value):
		child_list = self.get_children_names(bpy.context)
		self["child_name"] = child_list[value][0]

	slot_name_select = bpy.props.EnumProperty(
		items = get_materials_by_slot,
		get = get_matname,
		set = set_matname
	)
	slot_num = bpy.props.IntProperty(min = 0)
	slot_id = bpy.props.IntProperty(min = 0)
	
	child_name = bpy.props.EnumProperty(
		items = get_children_names,
		get = get_child_name,
		set = set_child_name
	)

	@classmethod
	def register(cls):
		TFMatConfigSetting.slots = bpy.props.CollectionProperty(
						type = cls,
					)
		TFMatConfigSetting.slots_activeidx = bpy.props.IntProperty(
						min = 0,
					)

	@classmethod
	def unregister(cls):
		del TFMatConfigSetting.slots
		del TFMatConfigSetting.slots_activeidx

class TFMatConfigSlotItemUIList(bpy.types.UIList):
	def draw_item(self, context, layout, data, item, icon, active_data, active_propname, index):
		if hasattr(data.id_data, "materials"):
			layout.prop(item, "slot_name_select", "Submesh " + str(index))
		elif item.child_name != "":
			row = layout.row()
			row.prop(item, "child_name", "Child name")
			row.prop(item, "slot_name_select", "MatConfig")
		else:
			layout.prop(item, "slot_name_select", "Slot " + str(index))

class TFMatConfigSetting(bpy.types.PropertyGroup):
	def slots_to_list(self):
		if hasattr(self.id_data, "data"):
			slot_list = [0 for i in range(len(self.id_data.children))]
		else:
			slot_list = [0 for i in range(len(self.id_data.materials))]
		i = 0
		for slot in self.slots:
			#mat_list = get_materials_by_slot(slot, bpy.context)
			index = i
			if hasattr(slot.id_data, "children"):
				if "child_name" in slot and slot["child_name"] in bpy.data.objects and bpy.data.objects[slot["child_name"]] in slot.id_data.children:
					index = slot.id_data.children.index(bpy.data.objects[slot["child_name"]])
			if "slot_name" in slot:
				matConfig = get_mat_index_by_slot(slot, slot["slot_name"])
			else:
				matConfig = 0
			if index < len(slot_list):
				slot_list[index] = matConfig
			i += 1
		return slot_list

	matConfig_name = bpy.props.StringProperty()

	@classmethod
	def register(cls):
		bpy.types.Mesh.tf_matConfigs = bpy.props.CollectionProperty(
						type = cls,
					)
		bpy.types.Mesh.tf_matConfigs_activeidx = bpy.props.IntProperty(
						min = 0,
					)
		bpy.types.Mesh.tf_matConfigs_slotremoveidx = bpy.props.IntProperty(
						min = -1,
						default = -1,
						name = "SlotID",
						description = "ID of slot to remove (-1: last)"
					)
		bpy.types.Mesh.tf_matConfig_id = bpy.props.IntProperty()
		bpy.types.Object.tf_matConfigs = bpy.props.CollectionProperty(
						type = cls,
					)
		bpy.types.Object.tf_matConfigs_activeidx = bpy.props.IntProperty(
						min = 0,
					)
		bpy.types.Object.tf_matConfigs_slotremoveidx = bpy.props.IntProperty(
						min = -1,
						default = -1,
						name = "SlotID",
						description = "ID of slot to remove (-1: last)"
					)
		bpy.types.Object.tf_matConfig_id = bpy.props.IntProperty()

	@classmethod
	def unregister(cls):
		del bpy.types.Mesh.tf_matConfigs
		del bpy.types.Mesh.tf_matConfigs_activeidx
		del bpy.types.Mesh.tf_matConfigs_slotremoveidx
		del bpy.types.Object.tf_matConfigs
		del bpy.types.Object.tf_matConfigs_activeidx
		del bpy.types.Object.tf_matConfigs_slotremoveidx

class TFMatConfigAddOp(bpy.types.Operator):
	bl_label = "Add matConfig property"
	bl_idname = "collection.tfmatconfig_add"

	hasdata = bpy.props.BoolProperty(default=False)

	def invoke(self, context, event):
		obj = context.object
		if self.hasdata:
			obj = obj.data
		collection = obj.tf_matConfigs
		item = collection.add()
		item.matConfig_name = "matConfig" + str(len(collection))
		if self.hasdata:
			for mat in obj.materials:
				slot = item.slots.add()
				slot.slot_id = len(item.slots)-1
		else:
			for child in obj.children:
				slot = item.slots.add()
				slot.slot_id = len(item.slots)-1
				slot.child_name = child.name
		obj.tf_matConfigs_activeidx = len(collection) - 1

		return {'FINISHED'}

class TFMatConfigRemoveOp(bpy.types.Operator):
	bl_label = "Remove matConfig property"
	bl_idname = "collection.tfmatconfig_remove"

	hasdata = bpy.props.BoolProperty(default=False)

	def invoke(self, context, event):
		obj = context.object
		if self.hasdata:
			obj = obj.data
		collection = obj.tf_matConfigs
		index = obj.tf_matConfigs_activeidx
		collection.remove(index)
		if index > 0:
			obj.tf_matConfigs_activeidx = index-1
		return {'FINISHED'}

class TFMatConfigSlotsAddOp(bpy.types.Operator):
	bl_label = "Add slots to matConfig"
	bl_idname = "collection.tfmatconfigslots_add"

	hasdata = bpy.props.BoolProperty(default=False)

	def invoke(self, context, event):
		obj = context.object
		if self.hasdata:
			obj = obj.data
			for matConfig in obj.tf_matConfigs:
				if len(obj.materials) > len(matConfig.slots):
					for i in range(len(matConfig.slots), len(obj.materials)):
						item = matConfig.slots.add()
						item.slot_id = i
		else:
			for matConfig in obj.tf_matConfigs:
				if len(obj.children) > len(matConfig.slots):
					for i in range(len(matConfig.slots), len(obj.children)):
						item = matConfig.slots.add()
						item.slot_id = i

		return {'FINISHED'}

class TFMatConfigSlotRemoveOp(bpy.types.Operator):
	bl_label = "Remove slot from matConfig"
	bl_idname = "collection.tfmatconfigslot_remove"

	hasdata = bpy.props.BoolProperty(default=False)

	def invoke(self, context, event):
		obj = context.object
		if self.hasdata:
			obj = obj.data
			index = obj.tf_matConfigs_slotremoveidx
			for matConfig in obj.tf_matConfigs:
				if len(obj.materials) < len(matConfig.slots):
					if index >= len(matConfig.slots) or index == -1:
						index = len(matConfig.slots) - 1
					matConfig.slots.remove(index)
					for i in range(index, len(matConfig.slots)):
						matConfig.slots[i].slot_id = i
		else:
			index = obj.tf_matConfigs_slotremoveidx
			for matConfig in obj.tf_matConfigs:
				if len(obj.children) < len(matConfig.slots):
					if index >= len(matConfig.slots) or index == -1:
						index = len(matConfig.slots) - 1
					matConfig.slots.remove(index)
					for i in range(index, len(matConfig.slots)):
						matConfig.slots[i].slot_id = i

		return {'FINISHED'}

class TFMatConfigSettingsItemUIList(bpy.types.UIList):
	def draw_item(self, context, layout, data, item, icon, active_data, active_propname, index):
		row = layout.row()
		row.prop(data.tf_matConfigs[index], "matConfig_name", "Name (internal)")
		row.template_list(
			listtype_name = "TFMatConfigSlotItemUIList",
			list_id = data.name + "MatConfigSlot" + str(index),
			dataptr = data.tf_matConfigs[index],
			propname = "slots",
			active_dataptr = data.tf_matConfigs[index],
			active_propname = "slots_activeidx",
			rows = 1,
			maxrows = 4
		)

# Plugin class
class TFMaterialImportExport():
	## Internal functions
	# Panel drawing functions
	def __draw_matConfigs(self, layout, mydata, attr=""):
		boxMatConfig = TF_base_functions.draw_hideable_box(layout, "Material Configs:", "mat_configs")
		if boxMatConfig:
			row = boxMatConfig.row()
			row.template_list(
				listtype_name="TFMatConfigSettingsItemUIList",
				dataptr=mydata,
				propname="tf_matConfigs",
				active_dataptr=mydata,
				active_propname="tf_matConfigs_activeidx",
				rows = 1,
				maxrows = 2
			)
			col = row.column(align=True)
			col.operator("collection.tfmatconfig_add", icon="ZOOMIN", text="").hasdata = (attr=="data")
			col.operator("collection.tfmatconfig_remove", icon="ZOOMOUT", text="").hasdata = (attr=="data")
			slots_needed = False
			tomuch_slots = False
			for matConfig in mydata.tf_matConfigs:
				slots_needed = slots_needed or (len(matConfig.slots) < get_slot_num(mydata))
				tomuch_slots = tomuch_slots or (len(matConfig.slots) > get_slot_num(mydata))

			if slots_needed:
				boxMatConfig.label("The number of slots for some matConfigs is lower than the number of materials for this mesh, use this operator to add slots where necessary.")
				boxMatConfig.operator("collection.tfmatconfigslots_add", icon="ZOOMIN", text="").hasdata = (attr=="data")
			if tomuch_slots:
				boxMatConfig.label("The number of slots for some matConfigs is higher than the number of materials for this mesh, use this operator to remove a given slot.")
				row = boxMatConfig.row()
				row.prop(mydata, "tf_matConfigs_slotremoveidx")
				row.operator("collection.tfmatconfigslot_remove", icon="ZOOMOUT", text="").hasdata = (attr=="data")
			layout.separator()

	def __draw_addMaterials(self, layout, mydata):
		boxMat = TF_base_functions.draw_hideable_box(layout, "Additional materials:", "add_materials")
		if boxMat:
			row = boxMat.row()
			row.template_list(
				listtype_name="TFMatSlotSettingsItemUIList",
				dataptr=mydata,
				propname="tf_matslots",
				active_dataptr=mydata,
				active_propname="tf_matslots_activeidx",
				rows = 1,
				maxrows = 2
			)
			col = row.column(align=True)
			col.operator("collection.tfmatslot_add", icon="ZOOMIN", text="")
			col.operator("collection.tfmatslot_remove", icon="ZOOMOUT", text="")
			layout.separator()

	def __load_matConfigs_parent(self, obj, data):
		for matConfig in data["matConfigs"]:
			item = obj.tf_matConfigs.add()
			item.matConfig_name = "matConfig" + str(len(obj.tf_matConfigs))
			for i in range(len(matConfig)):
				slot = item.slots.add()
				slot.slot_id = i
				child = None
				for j in range(len(obj.children)):
					if i < len(data["children"]) and "blender_object" in data["children"][i]:
						child = data["children"][i]["blender_object"]
				
				if child:
					slot.child_name = child.name
					if child.type == 'MESH':
						if matConfig[i] < len(child.data.tf_matConfigs):
							slot.slot_name_select = child.data.tf_matConfigs[matConfig[i]].matConfig_name
					else:
						if matConfig[i] < len(child.tf_matConfigs):
							slot.slot_name_select = child.tf_matConfigs[matConfig[i]].matConfig_name

	def __save_matConfigs_parent(self, obj, data):
		if not "matConfigs" in data:
			data["matConfigs"] = []
		for matConfig in obj.tf_matConfigs:
			slot_config = matConfig.slots_to_list()
			if not slot_config in data["matConfigs"]:
				data["matConfigs"].append(slot_config)
		if len(data["matConfigs"]) == 0:
			data["matConfigs"] = [[0 for i in range(len(data["children"]))]]

	## External functions
	# UI functions
	def draw_mesh_settings_panel(self, data):
		if TF_base_functions.user_prefs["debug"]:
			print("TFMaterialImportExport draw_mesh_settings_panel")
		self.__draw_addMaterials(data["layout"], data["mydata"])
		self.__draw_matConfigs(data["layout"], data["mydata"], "data")

	def draw_group_settings_panel(self, data):
		if TF_base_functions.user_prefs["debug"]:
			print("TFMaterialImportExport draw_group_settings_panel")
		self.__draw_matConfigs(data["layout"], data["mydata"])

	def draw_lod_settings_panel(self, data):
		if TF_base_functions.user_prefs["debug"]:
			print("TFMaterialImportExport draw_lod_settings_panel")
		self.__draw_matConfigs(data["layout"], data["mydata"])

	# import/export functions
	def save_mesh_data_once(self, data):
		if TF_base_functions.user_prefs["debug"]:
			print("TFMaterialImportExport save_mesh_data_once")

		obj = data["meshObj"]
		errors = []
		for j in range(len(obj.data.tf_matslots)):
			mat_slot = obj.data.tf_matslots[j]
			i = obj.data.tf_matslots[j].slot_index 

			if TF_base_functions.export_settings["game"] != "TPF2" and i < len(data["meshData"]["subMeshes"]):
				submesh = data["meshData"]["subMeshes"][i]
				materials = []
				for mat in mat_slot.tf_mats:
					if mat.mat_name in bpy.data.materials:
						mat_fileid = material_util.create_file_data(
							{"object": bpy.data.materials[mat.mat_name], "parent": {"object": obj, "model_type": data["modelType"]}},
							"",
							errors
						)
						if mat_fileid not in materials:
							materials.append(mat_fileid)
				
				for mat in submesh["materials"]:
					if mat not in materials:
						materials = [mat] + materials
				
				submesh["materials"] = materials

		if TF_base_functions.export_settings["game"] != "TPF2":
			for matConfig in data["meshObj"].data.tf_matConfigs:
				slot_config = matConfig.slots_to_list()
				data["meshData"]["matConfigs"].append([slot_config[i] for i in data["meshData"]["__binary"]["mat_slots"]])
			if len(data["meshData"]["matConfigs"]) == 0:
				data["meshData"]["matConfigs"] = [[0 for i in range(len(data["meshData"]["subMeshes"]))]]

		return errors

	def save_group_data(self, data):
		if TF_base_functions.user_prefs["debug"]:
			print("TFMaterialImportExport save_group_data")

		self.__save_matConfigs_parent(data["groupObj"], data["groupData"])

	def save_lod_data(self, data):
		if TF_base_functions.export_settings["game"] == "TPF2":
			return
			
		if TF_base_functions.user_prefs["debug"]:
			print("TFMaterialImportExport save_lod_data")

		if data["lodObj"]:
			self.__save_matConfigs_parent(data["lodObj"], data["lodData"])
					
	def load_mesh_after_creation(self, data):
		if TF_base_functions.user_prefs["debug"]:
			print("TFMaterialImportExport load_mesh_after_creation")

		material_data = data["materialData"]
		mesh = data["meshObj"].data
		for i in range(len(material_data)):
			if TF_base_functions.import_settings["game"] == "TPF2":
				mat_slot = mesh.tf_matslots.add()
				mat_slot.slot_index = i
				collection = mat_slot.tf_mats
				material = material_data[i]
				if not material.name in collection.keys():
					item = collection.add()
					item.mat_name = material.name
					item.name = material.name
			else:
				if len(material_data[i]) > 1:
					mat_slot = mesh.tf_matslots.add()
					mat_slot.slot_index = i
					collection = mat_slot.tf_mats
					for material in material_data[i]:
						if not material.name in collection.keys():
							item = collection.add()
							item.mat_name = material.name
							item.name = material.name

		if TF_base_functions.import_settings["game"] != "TPF2":
			for matConfig in data["meshData"]["matConfigs"]:
				item = mesh.tf_matConfigs.add()
				item.matConfig_name = "matConfig" + str(len(mesh.tf_matConfigs))
				for i in range(len(matConfig)):
					slot = item.slots.add()
					slot.slot_id = i
					slot.slot_name_select = material_data[i][matConfig[i]].name[:63]

	def load_mesh_always(self, data):
		if TF_base_functions.user_prefs["debug"]:
			print("TFMaterialImportExport load_mesh_always")
			
		obj = data["meshObj"]
		if obj and obj.data:
			for i in range(len(obj.material_slots)):
				mat_slot = None
				for j in range(len(obj.data.tf_matslots)):
					if obj.data.tf_matslots[j].slot_index == i:
						mat_slot = obj.data.tf_matslots[j]

				if mat_slot:
					if len(mat_slot.tf_mats) == 1:
						obj.material_slots[i].link = 'DATA'
						obj.material_slots[i].material = obj.data.materials[i]
					else:
						obj.material_slots[i].link = 'OBJECT'
						mat_name = mat_slot.tf_mats[data["meshData"]["matConfigs"][data["matConfig"]][i]].mat_name
						if mat_name in bpy.data.materials:
							obj.material_slots[i].material = bpy.data.materials[mat_name]

	def load_group(self, data):
		if TF_base_functions.user_prefs["debug"]:
			print("TFMaterialImportExport load_group")

		self.__load_matConfigs_parent(data["groupObj"], data["groupData"])

	def load_lod(self, data):
		if TF_base_functions.user_prefs["debug"]:
			print("TFMaterialImportExport load_lod")

		if TF_base_functions.import_settings["game"] != "TPF2" and data["lodObj"]:
			self.__load_matConfigs_parent(data["lodObj"], data["lodData"])

class_list = [
	TFMatSlotSetting,
	TFMatSlotAddOp,
	TFMatSlotRemoveOp,
	TFMatSlotSettingsItemUIList,
	TFMatSetting,
	TFMatAddOp,
	TFMatRemoveOp,
	TFMatSettingsItemUIList,
	TFMatConfigSlot,
	TFMatConfigSlotItemUIList,
	TFMatConfigSetting,
	TFMatConfigAddOp,
	TFMatConfigRemoveOp,
	TFMatConfigSlotsAddOp,
	TFMatConfigSlotRemoveOp,
	TFMatConfigSettingsItemUIList
]

def register():
	for class_type in class_list:
		bpy.utils.register_class(class_type)

	importexport = TFMaterialImportExport()
	TF_base_functions.plugin_add("TFMaterialImportExport", importexport)


def unregister():
	TF_base_functions.plugin_remove("TFMaterialImportExport")

	for i in range(len(class_list)-1, -1, -1):
		bpy.utils.unregister_class(class_list[i])