import bpy
import subprocess
from .base_functions import TF_base_functions
from os import remove
from os.path import isfile

def flip_image(image):
	pixels = image.pixels[:]
	width = image.size[0]
	height = image.size[1]
	new_pixels = []
	for i in range(height-1, -1, -1):
		new_pixels += pixels[4*i*width:4*(i+1)*width]
	image.pixels = new_pixels

def combine_images(back_image, front_image, offset, name):
	pixel_arr = list(back_image.pixels)
	front_pixels = front_image.pixels[:]

	back_width = back_image.size[0]
	back_height = back_image.size[1]
	front_width = front_image.size[0]
	front_height = front_image.size[1]
	front_width_orig = front_width
	if (front_width + offset[0] > back_width):
		front_width = back_width - offset[0]
	if (front_height + offset[1] > back_height):
		front_height = back_height - offset[1]

	for i in range(front_width):
		for j in range(front_height):
			back_index = 4*((j+offset[1])*back_width+i+offset[0])
			front_index = 4*(j*front_width_orig+i)
			front_alpha = front_pixels[front_index+3]
			pixel_arr[back_index] = pixel_arr[back_index]*(1-front_alpha) + front_pixels[front_index]*front_alpha
			pixel_arr[back_index+1] = pixel_arr[back_index+1]*(1-front_alpha) + front_pixels[front_index+1]*front_alpha
			pixel_arr[back_index+2] = pixel_arr[back_index+2]*(1-front_alpha) + front_pixels[front_index+2]*front_alpha
			pixel_arr[back_index+3] = 1.0-((1.0-pixel_arr[back_index+3])*(1.0-front_alpha))
	new_image = bpy.data.images.new(name, width=back_width, height=back_height)
	new_image.pixels = pixel_arr
	return new_image

def write_file(file, errors):
	if TF_base_functions.export_settings["textures"] in ("TGA", "DDS"):
		image = file["image"].copy()
		image.use_alpha = True
		if TF_base_functions.export_settings["textures"] == "DDS":
			flip_image(image)
			
		if image.channels == 0:
			errors.append({"level": 1, "message": "Skipping output of %s (blender is cray about AO format)." % TF_base_functions.export_settings["basepath"] + file["path"]})
		else:
			image.save_render(TF_base_functions.export_settings["basepath"] + file["path"], scene=TF_base_functions.export_settings["render_scene"])
		
		if TF_base_functions.export_settings["textures"] == "DDS" and isfile(TF_base_functions.dds_converter):
			if "_normal" in file["textureType"]:
				if file["matType"][:4] == "PHYS":
					subprocess.call([TF_base_functions.dds_converter, "-normal", "-bc5", TF_base_functions.export_settings["basepath"] + file["path"]])
			elif file["textureType"] in {"map_albedo", "map_metal_gloss_ao", "map_cblend_dirt_rust"}:
				subprocess.call([TF_base_functions.dds_converter, TF_base_functions.export_settings["basepath"] + file["path"]])
			else:
				subprocess.call([TF_base_functions.dds_converter, "-bc3", TF_base_functions.export_settings["basepath"] + file["path"]])
			
			# remove tga
			remove(TF_base_functions.export_settings["basepath"] + file["path"])
		
		image.user_clear()
		bpy.data.images.remove(image)
		del image

		TF_base_functions.export_settings["filelist"].append([TF_base_functions.export_settings["basepath"] + file["path"], file["path"]])